/*
 * s390-tools/zipl/include/boot.h
 *   Functions to handle the boot loader data.
 *
 * Copyright (C) 2001-2003 IBM Deutschland Entwicklung GmbH, IBM Corporation
 *
 * Author(s): Carsten Otte <cotte@de.ibm.com>
 *            Peter Oberparleiter <Peter.Oberparleiter@de.ibm.com>
 */

#ifndef BOOT_H
#define BOOT_H

#include "zipl.h"

#include <sys/types.h>

#include "disk.h"


/* Boot data structures for FBA disks */

struct boot_fba_locread {
	uint64_t locate;
	uint64_t read;
} __attribute__ ((packed));

struct boot_fba_locdata {
	uint8_t command;
	uint8_t dummy;
	uint16_t blockct;
	uint32_t blocknr;
} __attribute__ ((packed));

struct boot_fba_stage0 {
	uint64_t PSW;
	uint64_t READ;
	uint64_t TIC;
	uint64_t param1;
	uint64_t param2;
	struct boot_fba_locread locread[8];
	struct boot_fba_locdata locdata[8];
} __attribute__ ((packed));


/* Boot data structures for ECKD disks */

struct boot_eckd_ccw0 {
	uint8_t cmd;
	uint8_t address_hi;
	uint16_t address_lo;
	uint8_t flags;
	uint8_t pad;
	uint16_t count;
} __attribute__ ((packed));

struct boot_eckd_ccw1 {
	uint8_t cmd;
	uint8_t flags;
	uint16_t count;
	uint32_t address;
} __attribute__ ((packed));

struct boot_eckd_ssrt {
	struct boot_eckd_ccw0 seek;
	struct boot_eckd_ccw0 search;
	struct boot_eckd_ccw0 tic;
	struct boot_eckd_ccw0 read;
} __attribute__ ((packed));

struct boot_eckd_seekarg {
	uint16_t pad;
	uint16_t cyl;
	uint16_t head;
	uint8_t sec;
	uint8_t pad2;
} __attribute__ ((packed));

struct boot_eckd_stage0 {
	uint64_t psw;
	struct boot_eckd_ccw0 read;
	struct boot_eckd_ccw0 tic;
} __attribute__ ((packed)) ipleckd0_t;

struct boot_eckd_classic_stage1 {
	uint64_t param1;
	uint64_t param2;
	struct boot_eckd_ssrt ssrt[8];
	struct boot_eckd_seekarg seek[8];
} __attribute__ ((packed));

struct boot_eckd_compatible_stage1 {
	uint64_t param1;
	uint64_t param2;
	struct boot_eckd_ccw0 seek;
	struct boot_eckd_ccw0 search;
	struct boot_eckd_ccw0 tic;
	struct boot_eckd_ccw0 read[12];
	struct boot_eckd_seekarg seekarg;
} __attribute__ ((packed)) ipleckd1b_t;


int boot_check_data(void);
int boot_init_fba_stage0(struct boot_fba_stage0* stage0,
			 disk_blockptr_t* program_table,
			 disk_blockptr_t* stage2_list,
			 blocknum_t stage2_count, struct disk_info* info);
int boot_get_fba_stage2(void** data, size_t* size);
void boot_init_eckd_classic_stage0(struct boot_eckd_stage0* stage0);
int boot_init_eckd_classic_stage1(struct boot_eckd_classic_stage1* stage1,
				  disk_blockptr_t* stage2_list, blocknum_t stage2_count,
				  struct disk_info* info);
void boot_init_eckd_compatible_stage0(struct boot_eckd_stage0* stage0);
int boot_init_eckd_compatible_stage1(
			struct boot_eckd_compatible_stage1* stage1,
			struct disk_info* info);
int boot_get_eckd_stage2(void** data, size_t* size);
int boot_get_stage3(void** buffer, size_t* bytecount, address_t parm_addr,
		    address_t initrd_addr, size_t initrd_len,
		    address_t image_addr);
int boot_get_tape_dump(void** data, size_t* size);
int boot_get_eckd_dump_stage2(void** data, size_t* size);
int boot_init_eckd_classic_dump_stage1(struct boot_eckd_classic_stage1* stage1,
				       struct disk_info* info);
int boot_init_eckd_compatible_dump_stage1(
				struct boot_eckd_compatible_stage1* stage1,
				struct disk_info* info);

#endif /* BOOT_H */
